# -*- coding:utf-8 -*-

#__________________________________________________________________________________________
# Extension   : MPMA LibreOffice Addon
# Version     : 1.0.8 - in 13-Jan-2020
# Modules     : LibreOffice v2.0.x
# Objective   : Utilities to write an extended value
# Author      : Tibério Augusto Lima de Melo
#__________________________________________________________________________________________
# Class       : FullValue
# Update      : 12-Jul-2017
#__________________________________________________________________________________________
# LICENSE     : GPL 3.0
#
#  Copyright (C) 2018 Tibério Augusto Lima de Melo
#  
#  This file is part of MPMA LibreOffice Addon.
#  
#  MPMA LibreOffice Addon is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#   
#  MPMA LibreOffice Addon is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with MPMA LibreOffice Addon.  If not, see <http://www.gnu.org/licenses/>.
#__________________________________________________________________________________________

class FullValue(object):
  #**************************************************************************************
  # Constants
  #**************************************************************************************
  IDENTIFIER  = {'DECIMAL':",", 'PERCENTUAL':"%", 'THOUSAND': ".", 'CURRENCY':"R$"}
  DESCRIPTOR  = {'HUNDRED':"cem", 'ONE':"um", 'TEN':"dez", 'ZERO':"zero"}
  LABEL       = {'CENT':" centavo ", 'CENTS':" centavos ", 'CURRENCY_SINGULAR':" real ", 'CURRENCY_PLURAL':" reais ", 'PERCENTUAL':" por cento "}
  SEPARATOR   = {'DECIMAL':" virgula ", 'PERCENTUAL':" ponto ", 'THOUSAND':", ", 'UNIT':" e ", 'CURRENCY':"e "}
  KIND        = {'CURRENCY': 0, 'PERCENTUAL':1, 'NUMBER':2}
  ERROR       = "## erro ##"
  #**************************************************************************************
  # Full Values
  #**************************************************************************************
  FV_UNIT     = ("", DESCRIPTOR['ONE'], "dois", "três", "quatro", "cinco", "seis", "sete", "oito", "nove",  "dez")
  FV_TEN      = ("", "dez", "vinte", "trinta", "quarenta", "cinquenta", "sessenta", "setenta", "oitenta", "noventa")
  FV_TENTH    = ("", "onze", "doze", "treze", "quatorze", "quinze", "dezesseis", "dezessete", "dezoito", "dezenove")
  FV_HUNDRED  = ("", "cento", "duzentos", "trezentos", "quatrocentos", "quinhentos", "seiscentos", "setecentos", "oitocentos", "novecentos")
  FV_THOUSAND_SINGULAR  = ("", " mil", " milhão", " bilhão", " trilhão", " quatrilhão")
  FV_THOUSAND_PLURAL    = ("", " mil", " milhões", " bilhões", " trilhões", " quatrilhões")
    
  def __init__(self, number):
    self.number = str(number)

  def setValue(self, number):
    self.number = str(number)
    
  def getValue(self):
    return self.number

  def isValid(self, number = None):
    #**************************************************************************************
    # isValid     : Informs is character if valid for fullvalue identification
    # Author      : Tibério Melo
    # Update      : 28-Jun-2017
    #**************************************************************************************
    if number is None: number = self.number
    return ("0123456789"+self.IDENTIFIER['DECIMAL']+self.IDENTIFIER['PERCENTUAL']+self.IDENTIFIER['THOUSAND']).find(str(number)[0])>=0
          
  def lblUnit(self, number = None):
    #**************************************************************************************
    # lblUnit     : Build the extension of the unit
    # Author      : Tibério Melo
    # Update      : 13-Apr-2017
    #**************************************************************************************
    if number is None: number = self.number
    if (str(number).isdigit() or str(number)==""):
      unit = int(number) % 10
      return self.FV_UNIT[unit]
    else:
      return self.ERROR
        
  def lblTen(self, number = None):
    #**************************************************************************************
    # lblTen      : Build the extension of the ten
    # Author      : Tibério Melo
    # Update      : 13-Apr-2017
    #**************************************************************************************
    if number is None: number = self.number
    if (str(number).isdigit() or str(number)==""):
      ten = int(number) % 100
      if ten < 10:
        return self.lblUnit(number)
      elif ten == 10:
        return self.DESCRIPTOR['TEN']
      elif ten < 20:
        return self.FV_TENTH[ten-10]
      else:
        return (self.FV_TEN[int(ten/10)]+(self.SEPARATOR['UNIT'],"")[self.lblUnit(number)==""]+self.lblUnit(number))
    else:
      return self.ERROR

  def lblHundred(self, number = None):
    #**************************************************************************************
    # lblHundred  : Build the extension of the hundred
    # Author      : Tibério Melo
    # Update      : 13-Apr-2017
    #**************************************************************************************
    if number is None: number = self.number
    if (str(number).isdigit() or str(number)==""):
      hundred = int(number) % 1000
      if hundred < 100:
        return self.lblTen(number)
      elif hundred == 100:
        return self.DESCRIPTOR['HUNDRED']
      else:
        return (self.FV_HUNDRED[int(hundred/100)]+(self.SEPARATOR['UNIT'],"")[self.lblTen(number)==""]+self.lblTen(number))
    else:
      return self.ERROR
      
  def lblNumber(self, number = None):
    #**************************************************************************************
    # lblNumber   : Build the extension of the number
    # Author      : Tibério Melo
    # Update      : 13-Apr-2017
    #**************************************************************************************
    if number is None: number = self.number
    number = str(number)
    if (number.isdigit() or number==""):
      #-------------------------    
      # inicialize variables
      #-------------------------
      count_power=0
      count_user=0
      full=""
      separator=self.SEPARATOR['UNIT']
      #-------------------------      
      # build full value
      #-------------------------      
      for count in range(len(number),0,-3):
        if count<3:
          start=0
          step=(len(number) % 3)
        else:
          start=count-3
          step=3
        iNumber=int(number[start:start+step])
        hundred=self.lblHundred(iNumber)
        if iNumber == 1:
          full=hundred+self.FV_THOUSAND_SINGULAR[count_power]+(separator,"")[full==""]+full
        elif iNumber > 1:
          full=hundred+self.FV_THOUSAND_PLURAL[count_power]+(separator,"")[full==""]+full
        if hundred != "": count_user += 1
        if count_user == 2: separator=self.SEPARATOR['THOUSAND']
        count_power += 1
      return full
    else:
      return self.ERROR
      
  def split(self, kind, number = None):
    #**************************************************************************************
    # split       : Split number in decimal and fractionary part
    # Author      : Tibério Melo
    # Update      : 14-Apr-2017
    #**************************************************************************************
    if number is None: number = self.number
    sNumber=str(number).replace(self.IDENTIFIER['THOUSAND'], "")
    sNumber=sNumber.strip()
    if (sNumber.replace(self.IDENTIFIER['DECIMAL'],'',1).isdigit() or sNumber==""):
      pos=sNumber.find(self.IDENTIFIER['DECIMAL'])
      if pos==-1:
        sInteger=sNumber
        sDecimal=""
      else:
        sInteger=sNumber[:pos]
        sDecimal=sNumber[pos+1:]
      #-------------------------
      # Split by kind
      #-------------------------      
      if kind == self.KIND['CURRENCY']:
        sDecimal=sDecimal[:2]
        if len(sDecimal)==1: sDecimal+="0"
      elif kind == self.KIND['PERCENTUAL']:
        sDecimal=sDecimal[:4]      
        sDecimal+="0"*(4-len(sDecimal))
        sInteger+=sDecimal[:2]
        sDecimal=sDecimal[2:4]
      #-------------------------
      # Make labels
      #-------------------------        
      lblInteger=self.lblNumber(sInteger)
      if lblInteger=="": lblInteger=self.DESCRIPTOR['ZERO']
      lblDecimal=self.lblNumber(sDecimal)
      return [lblInteger, lblDecimal]
    else:
      return [self.ERROR, self.ERROR]

  def toCurrency(self, number = None):
    #**************************************************************************************
    # toCurrency  : Return full value like currency
    # Author      : Tibério Melo
    # Update      : 14-Apr-2017
    #**************************************************************************************
    if number is None: number = self.number
    if (str(number).replace(self.IDENTIFIER['THOUSAND'], "").replace(self.IDENTIFIER['DECIMAL'],'',1).isdigit() or str(number)==""):
      labels=self.split(self.KIND['CURRENCY'], number)
      currency=""
      if labels[0]!=self.DESCRIPTOR['ZERO']:
        currency = labels[0]
        currency+= (self.LABEL['CURRENCY_PLURAL'],self.LABEL['CURRENCY_SINGULAR'])[labels[0]==self.DESCRIPTOR['ONE']]
        currency+= (self.SEPARATOR['CURRENCY'],"")[labels[1]==""]
      currency+=labels[1]
      if labels[1]!="":
        currency+=(self.LABEL['CENTS'],self.LABEL['CENT'])[labels[1]==self.DESCRIPTOR['ONE']]
      return currency.strip()
    else:
      return self.ERROR      

  def toPercentual(self, number = None):
    #**************************************************************************************
    # toPercentual: Return full value like percentual
    # Author      : Tibério Melo
    # Update      : 14-Apr-2017
    #**************************************************************************************
    if number is None: number = self.number
    if (str(number).replace(self.IDENTIFIER['THOUSAND'], "").replace(self.IDENTIFIER['DECIMAL'],'',1).isdigit() or str(number)==""):
      labels=self.split(self.KIND['PERCENTUAL'], number)
      return (labels[0]+(self.SEPARATOR['PERCENTUAL'],"")[labels[1]==""]+labels[1]+self.LABEL['PERCENTUAL']).strip()
    else:
      return self.ERROR
	    
  def toNumber(self, number = None):
    #**************************************************************************************
    # toPercentual: Return full value like number
    # Author      : Tibério Melo
    # Update      : 14-Apr-2017
    #**************************************************************************************
    if number is None: number = self.number
    if (str(number).replace(self.IDENTIFIER['THOUSAND'], "").replace(self.IDENTIFIER['DECIMAL'],'',1).isdigit() or str(number)==""):
      labels=self.split(self.KIND['NUMBER'], number)
      return (labels[0]+(self.SEPARATOR['DECIMAL'],"")[labels[1]==""]+labels[1]).strip()
    else:
      return self.ERROR

  def toFull(self, number = None):
    #**************************************************************************************
    # toFull      : Return full value according keys
    # Author      : Tibério Melo
    # Update      : 14-Apr-2017
    #--------------------------------------------------------------------------------------
    # Keys:
    # a) R$ ou r$ at the beginning of the string  -> converts into currency
    # b) % ou %% at the end of the string         -> converts into percentual
    # c) Only numbers                             -> converts into number
    #**************************************************************************************
    if number is None: number = self.number
    sNumber=str(number).strip()
    #------------------------
    # check if is currency
    #------------------------
    if sNumber[:len(self.IDENTIFIER['CURRENCY'])].upper()==self.IDENTIFIER['CURRENCY'].upper():
      full = self.toCurrency(sNumber[len(self.IDENTIFIER['CURRENCY']):].strip())
    #------------------------
    # check if is percentual
    #------------------------  
    elif sNumber[-1:]==self.IDENTIFIER['PERCENTUAL']:
      size=1
      if (sNumber[-2:]==(self.IDENTIFIER['PERCENTUAL']+self.IDENTIFIER['PERCENTUAL'])): size=2
      percentual = sNumber[:-size].strip()
      pos = percentual.find(self.IDENTIFIER['DECIMAL'])
      if pos==-1:
        sInteger=percentual
        sDecimal=""
      else:
        sInteger=percentual[:pos]
        if sInteger=="": sInteger="0"
        sDecimal=percentual[pos+1:]		  
      if len(sInteger)>2:
        sInicio=sInteger[:-2]
      else:
        sInicio="0"
      percentual = sInicio + self.IDENTIFIER['DECIMAL'] + ("","0")[len(sInteger)==1] + sInteger[-2:] + sDecimal
      full = self.toPercentual(percentual)
    else:
      #------------------------
      # using number
      #------------------------      
      full = self.toNumber(sNumber)
    return full
