# -*- coding:utf-8 -*-

#__________________________________________________________________________________________
# Extension   : MPMA LibreOffice Addon
# Version     : 1.0.8 - in 13-Jan-2020
# Modules     : LibreOffice v2.0.x
# Objective   : Utilities to write an extended value
# Author      : Tibério Augusto Lima de Melo
#__________________________________________________________________________________________
# Class       : FullValueOO
# Update      : 31-Jan-2018
#__________________________________________________________________________________________
# LICENSE     : GPL 3.0
#
#  Copyright (C) 2018 Tibério Augusto Lima de Melo
#  
#  This file is part of MPMA LibreOffice Addon.
#  
#  MPMA LibreOffice Addon is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#   
#  MPMA LibreOffice Addon is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with MPMA LibreOffice Addon.  If not, see <http://www.gnu.org/licenses/>.
#__________________________________________________________________________________________

# Import
import uno
import unohelper
from br.mp.mpma.extension.fullvalue import FullValue as FullValue
from br.mp.mpma.extension.interfaces import XFullValue

#*******************************************************************************
# FullValueOO class
#*******************************************************************************
class FullValueOO(unohelper.Base, XFullValue):

  #-------------------------------------------------------------------------------
  # Constants
  #-------------------------------------------------------------------------------      
  # Service Name
  SERVICE_NAME =  "br.mp.mpma.extension.python.fullvalue"
  # List of Services
  SERVICE_NAMES = ( SERVICE_NAME, "com.sun.star.sheet.AddIn", )

  def __init__(self, context):
    #-------------------------------------------------------------------------------
    # Inicialize class
    # Author          : Tibério Melo
    # Update          : 31-Jan-2018
    #-------------------------------------------------------------------------------
    self.context = context
  
  def info(self, information):
    #-------------------------------------------------------------------------------
    # XFullValue Interface Method. Return information about fullvalue constants.
    # Author          : Tibério Melo
    # Update          : 31-Jan-2018
    #-------------------------------------------------------------------------------
    if    information.lower() == "decimal":
      return FullValue.IDENTIFIER['DECIMAL']
    elif    information.lower() == "percentual":
      return FullValue.IDENTIFIER['PERCENTUAL']
    elif    information.lower() == "thousand":
      return FullValue.IDENTIFIER['THOUSAND']
    elif    information.lower() == "currency":
      return FullValue.IDENTIFIER['CURRENCY']      
    elif  information.lower() == "error":
      return FullValue.ERROR
    else:
      return ""
  
  def isValid(self, character):
    #-------------------------------------------------------------------------------
    # XFullValue Interface Method. Inform if character is valid.
    # Author          : Tibério Melo
    # Update          : 31-Jan-2018
    #-------------------------------------------------------------------------------
    value=FullValue(character)
    return value.isValid()
    
  def toCurrency(self, number):
    #-------------------------------------------------------------------------------
    # XFullValue Interface Method. Convert value to currency fullvalue.
    # Author          : Tibério Melo
    # Update          : 31-Jan-2018
    #-------------------------------------------------------------------------------
    value=FullValue(number)
    return value.toCurrency()
    
  def toPercentual(self, number):
    #-------------------------------------------------------------------------------
    # XFullValue Interface Method. Convert value to percentual fullvalue.
    # Author          : Tibério Melo
    # Update          : 31-Jan-2018
    #-------------------------------------------------------------------------------  
    value=FullValue(number)
    return value.toPercentual()
    
  def toNumber(self, number):
    #-------------------------------------------------------------------------------
    # XFullValue Interface Method. Convert value to number fullvalue.
    # Author          : Tibério Melo
    # Update          : 31-Jan-2018
    #-------------------------------------------------------------------------------    
    value=FullValue(number)
    return value.toNumber()

  def toFull(self, number):
    #-------------------------------------------------------------------------------
    # XFullValue Interface Method. Convert value to fullvalue according type.
    # Author          : Tibério Melo
    # Update          : 31-Jan-2018
    #-------------------------------------------------------------------------------      
    value=FullValue(number)
    return value.toFull()

#*******************************************************************************
# Define services
#*******************************************************************************
g_ImplementationHelper = unohelper.ImplementationHelper()
g_ImplementationHelper.addImplementation(FullValueOO, FullValueOO.SERVICE_NAME, FullValueOO.SERVICE_NAMES, )
